unit CalendarUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Graphics, Windows, Dialogs, LazUTF8Classes,
  Forms;

type

  { TMyCalendar }

  TMyCalendar = class
  private
    FLang: Integer;
    FWeekNum: Boolean;
    FJpnFiscalYear: Boolean;
    FYear: WORD;
    procedure SetJpnFiscalYear(AValue: Boolean);
    procedure SetYear(AValue: WORD);
  public
    constructor Create(AYear: WORD);
    function MakeEMF(FileName: string = ''): THandle;
    procedure MakeEMFFile(FileName: string);
    property Year: WORD read FYear write SetYear;
    property Lang: Integer read FLang write FLang;
    property WeekNum: Boolean read FWeekNum write FWeekNum;
    property JpnFiscalYear: Boolean read FJpnFiscalYear write SetJpnFiscalYear;
  end;

var
  Holidays: TStringList;

const
  IS_HOLIDAY_NONE = 0;
  IS_HOLIDAY_SUNDAY = 1;
  IS_HOLIDAY_DAY = 2;
  IS_HOLIDAY_DAY_FURIKAE = 3;
  IS_HOLIDAY_EQUINOX = 4;
  IS_HOLIDAY_EQUINOX_FURIKAE = 5;
  IS_HOLIDAY_WEEK = 6; // 曜日指定の休日なので、振替休日は発生しない

//function IsHoliday(ADate: Int64; AHolidays: TStringList): Integer; overload;
function IsHoliday(ADate: Int64): Boolean; inline; overload;
procedure LoadHolidaysFromFile(AFileName: string);
procedure SaveHolidaysToFile(AFileName: string);

implementation

// 春分の日？
function IsVernalEquinox(wYear, wMonth, wDay: WORD): Boolean; inline;
begin
  Result := FALSE;
  if(wMonth = 3)and(wYear >= 1900)and(wYear <= 2099)then
  begin
    case wYear mod 4 of
    0:case wYear of
      1900..1956: Result := wDay = 21;
      1960..2088: Result := wDay = 20;
      2092..2096: Result := wDay = 19;
      end;
    1:case wYear of
      1901..1989: Result := wDay = 21;
      1993..2097: Result := wDay = 20;
      end;
    2:case wYear of
      1902..2022: Result := wDay = 21;
      2026..2098: Result := wDay = 20;
      end;
    3:case wYear of
      1903..1923: Result := wDay = 22;
      1927..2055: Result := wDay = 21;
      2059..2099: Result := wDay = 20;
      end;
    end;
  end;
end;

// 秋分の日？
function IsAutumnEquinox(wYear, wMonth, wDay: WORD): Boolean; inline;
begin
  Result := FALSE;
  if(wMonth = 9)and(wYear >= 1900)and(wYear <= 2099)then
  begin
    case wYear mod 4 of
    0:case wYear of
      1900..2008: Result := wDay = 23;
      2012..2096: Result := wDay = 22;
      end;
    1:case wYear of
      1901..1917: Result := wDay = 24;
      1921..2041: Result := wDay = 23;
      2045..2097: Result := wDay = 22;
      end;
    2:case wYear of
      1902..1946: Result := wDay = 24;
      1950..2074: Result := wDay = 23;
      2078..2098: Result := wDay = 22;
      end;
    3:case wYear of
      1903..1979: Result := wDay = 24;
      1983..2099: Result := wDay = 23;
      end;
    end;
  end;
end;
{
function IsHoliday(ADate: Int64; AHolidays: TStringList): Integer;
var
  wYear, wMonth, wDay, wDay_of_Week: WORD;
  wYear2, wMonth2, wDay2: WORD;
  I, iMonth, iDay: Integer;
begin
  Result := IS_HOLIDAY_NONE;

  DecodeDate(ADate, wYear, wMonth, wDay);
  wDay_of_Week := (ADate - 1) mod 7;
  // 0:日、1:月、2:火、3:水、4:木、5:金、6:土

  // 曜日でチェック（日曜日ならTRUE）
  if wDay_of_Week = 0 then
    Result := IS_HOLIDAY_SUNDAY;

  if Result = IS_HOLIDAY_NONE then
  begin
    if wDay_of_Week = 1 then // 月曜日なら振替休日の判定に備える
      DecodeDate(ADate - 1, wYear2, wMonth2, wDay2)
    else
    begin
      wYear2 := 0;
      wMonth2 := 0;
      wDay2 := 0;
    end;

    if IsVernalEquinox(wYear, wMonth, wDay) or IsAutumnEquinox(wYear, wMonth, wDay) then
      Result := IS_HOLIDAY_EQUINOX;
    if(Result = IS_HOLIDAY_NONE)and
      (wDay_of_Week = 1) then // 月曜日なので振替休日かも？
    if IsVernalEquinox(wYear2, wMonth2, wDay2) or IsAutumnEquinox(wYear2, wMonth2, wDay2) then
      Result := IS_HOLIDAY_EQUINOX_FURIKAE;

    if Result = IS_HOLIDAY_NONE then
    for I := 0 to AHolidays.Count - 1 do
    begin
      iMonth := Integer(AHolidays.Objects[I]);
      iDay := iMonth mod 100;
      iMonth := iMonth div 100;

      if (wMonth = iMonth) then
      begin
        if (wDay = iDay) then
          Result := IS_HOLIDAY_DAY
        else if iDay > 40 then
        begin
          if(((wDay + 6) div 7) = (iDay - 40) div 10) and // 第何週？
            (wDay_of_Week = (iDay mod 10)) then // 曜日？
            Result := IS_HOLIDAY_WEEK;
        end;
      end;

      // 月曜日は前日（日曜日）の振替休日になる可能性がある
      if(Result = IS_HOLIDAY_NONE)and
        (wDay_of_Week = 1)and
        (wMonth2 = iMonth)and(wDay2 = iDay)then
      begin
        Result := IS_HOLIDAY_DAY_FURIKAE;
      end;

      if Result <> IS_HOLIDAY_NONE then
        break;
    end;// for I := 0 to AHolidays.Count - 1 do
  end;
end;
}
function _IsHoliday(wYear, wMonth, wDay, wDay_of_Week, wYear2, wMonth2, wDay2: WORD; AHolidays: TStringList): Integer;
var
  I, iMonth, iDay: Integer;
begin
  Result := IS_HOLIDAY_NONE;
  {
  DecodeDate(ADate, wYear, wMonth, wDay);
  wDay_of_Week := (ADate - 1) mod 7;
  // 0:日、1:月、2:火、3:水、4:木、5:金、6:土
  }
  // 曜日でチェック（日曜日ならTRUE）
  if wDay_of_Week = 0 then
    Result := IS_HOLIDAY_SUNDAY;

  if Result = IS_HOLIDAY_NONE then
  begin
    {
    if wDay_of_Week = 1 then // 月曜日なら振替休日の判定に備える
    begin
      //DecodeDate(ADate - 1, wYear2, wMonth2, wDay2);
      DecodeDate(EncodeDate(wYear, wMonth, wDay) - 1, wYear2, wMonth2, wDay2);
    end
    else
    begin
      wYear2 := 0;
      wMonth2 := 0;
      wDay2 := 0;
    end;
    }
    if IsVernalEquinox(wYear, wMonth, wDay) or IsAutumnEquinox(wYear, wMonth, wDay) then
      Result := IS_HOLIDAY_EQUINOX;
    if(Result = IS_HOLIDAY_NONE)and
      (wDay_of_Week = 1) then // 月曜日なので振替休日かも？
    if IsVernalEquinox(wYear2, wMonth2, wDay2) or IsAutumnEquinox(wYear2, wMonth2, wDay2) then
      Result := IS_HOLIDAY_EQUINOX_FURIKAE;

    if Result = IS_HOLIDAY_NONE then
    for I := 0 to AHolidays.Count - 1 do
    begin
      iMonth := Integer(AHolidays.Objects[I]);
      iDay := iMonth mod 100;
      iMonth := iMonth div 100;

      if (wMonth = iMonth) then
      begin
        if (wDay = iDay) then
          Result := IS_HOLIDAY_DAY
        else if iDay > 40 then
        begin
          if(((wDay + 6) div 7) = (iDay - 40) div 10) and // 第何週？
            (wDay_of_Week = (iDay mod 10)) then // 曜日？
            Result := IS_HOLIDAY_WEEK;
        end;
      end;

      // 月曜日は前日（日曜日）の振替休日になる可能性がある
      if(Result = IS_HOLIDAY_NONE)and
        (wDay_of_Week = 1)and
        (wMonth2 = iMonth)and(wDay2 = iDay)then
      begin
        Result := IS_HOLIDAY_DAY_FURIKAE;
      end;

      if Result <> IS_HOLIDAY_NONE then
        break;
    end;// for I := 0 to AHolidays.Count - 1 do
  end;
end;

function IsHoliday(ADate: Int64): Boolean;
var
  wYear, wMonth, wDay, wDay_of_Week: WORD;
  wYear2, wMonth2, wDay2: WORD;
  AHolidays: TStringList;
  IObj: Integer;
  I, Index: Integer;
begin
  DecodeDate(ADate, wYear, wMonth, wDay);
  wDay_of_Week := (ADate - 1) mod 7;
  // 0:日、1:月、2:火、3:水、4:木、5:金、6:土

  if wDay_of_Week = 1 then // 月曜日なら振替休日の判定に備える
  begin
    DecodeDate(ADate - 1, wYear2, wMonth2, wDay2);
  end
  else
  begin
    wYear2 := 0;
    wMonth2 := 0;
    wDay2 := 0;
  end;

  //Result := IsHoliday(ADate, Holidays) <> IS_HOLIDAY_NONE;
  AHolidays := TStringList.Create;
  try
    for I := 0 to Holidays.Count - 1 do
    begin
      IObj := Integer(Holidays.Objects[I]);
      if IObj > 10000 then
      begin
        // ある年に特例の休日がある場合
        if (IObj div 10000) = wYear then
        begin
          Index := AHolidays.IndexOf(Holidays[I]);
          if Index >= 0 then
            AHolidays.Objects[Index] := TObject(IObj mod 10000);
        end;
      end
      else
        AHolidays.AddObject(Holidays[I], TObject(IObj));
    end;

    Result := _IsHoliday(
      wYear, wMonth, wDay, wDay_of_Week,
      wYear2, wMonth2, wDay2,
      AHolidays) <> IS_HOLIDAY_NONE;

  finally
    AHolidays.Free;
  end;
end;

function StrToHolidayValue(Src: string): Integer;
  function _WideStrToIntDef0(WS: WideString): Integer; inline;
  begin
    Result := StrToIntDef(Utf8Encode(WS), 0);
  end;
  function _AnsiToWideSting(S: AnsiString): WideString; inline;
  var
    L: longint;
  begin
    L := Windows.MultiByteToWideChar(932, 0, PChar(S), -1, nil, 0);
    SetLength(Result, L);
    Windows.MultiByteToWideChar(932, 0, PChar(S), -1, PWideChar(Result), L);
  end;
  procedure _Sub(WS: WideString);
  var
    SDayOfWeek: string;
    I1, I2: Integer;
    IMonth, IWeek, IDay, IDayOfWeek: Integer;
  begin
    I1 := Pos('月第', WS);
    I2 := Pos('曜', WS);
    if(I1 > 1)and(I2 = I1 + 4)then
    begin
      // "1月第2月曜日" など
      IMonth := _WideStrToIntDef0(Copy(WS, 1, I1 - 1));
      IWeek := _WideStrToIntDef0(Copy(WS, I1 + 2, 1));
      SDayOfWeek := UTF8Encode(Copy(WS, I1 + 3, 1));
      if SDayOfWeek = '日' then IDayOfWeek := 0
      else if SDayOfWeek = '月' then IDayOfWeek := 1
      else if SDayOfWeek = '火' then IDayOfWeek := 2
      else if SDayOfWeek = '水' then IDayOfWeek := 3
      else if SDayOfWeek = '木' then IDayOfWeek := 4
      else if SDayOfWeek = '金' then IDayOfWeek := 5
      else if SDayOfWeek = '土' then IDayOfWeek := 6
      else IDayOfWeek := -1;

      if(IMonth >= 1)and(IMonth <= 12)and
        (IWeek >= 1)and(IWeek <= 4)and
        (IDayOfWeek >= 0)and(IDayOfWeek <= 6)then
        Result := IMonth * 100 + IWeek * 10 + IDayOfWeek + 40;
    end
    else
    begin
      I1 := Pos('月', WS);
      I2 := Pos('日', WS);
      if(I1 > 1)and(I2 > I1 + 1)then
      begin
        IMonth := _WideStrToIntDef0(Copy(WS, 1, I1 - 1));
        IDay := _WideStrToIntDef0(Copy(WS, I1 + 1, I2 - I1 - 1));
        if(IMonth >= 1)and(IMonth <= 12)and
          (IDay >= 1)and(IDay <= 31)then
          Result := IMonth * 100 + IDay;
      end;
    end;
  end;
begin
  Result := StrToIntDef(Src, 0);
  if Result = 0 then
    _Sub(Utf8Decode(Src));
  if Result = 0 then
    // _Sub(Utf8Decode(AnsiToUtf8(Src))); // Lazarus 1.8.0 のバグ？
    _Sub(_AnsiToWideSting(Src));
end;

procedure LoadHolidaysFromFile(AFileName: string);
var
  SL: TStringList;
  I, Value: Integer;
begin
  SL := TStringList.Create;
  try
    SL.LoadFromFile(AFileName);

    for I := SL.Count - 1 downto 0 do
    begin
      Value := StrToHolidayValue(SL.ValueFromIndex[I]);

      if Value > 0 then
      begin
        SL[I] := SL.Names[I];
        SL.Objects[I] := TObject(Value);
      end
      else
        SL.Delete(I);
    end;// for I := SL.Count - 1 downto 0 do

    Holidays.Assign(SL);
  finally
    SL.Free
  end;
end;

procedure SaveHolidaysToFile(AFileName: string);
var
  I, Value: Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  try
    for I := 0 to Holidays.Count - 1 do
    begin
      Value := Integer(Holidays.Objects[I]);
      SL.Add(Holidays[I] + '=' + IntToStr(Value));
    end;
    SL.SaveToFile(AFileName);
  finally
    SL.Free
  end;
end;

function GetShortMonthName(Month: WORD; Lang: Integer = 0): string;
begin
  case Lang of
  1:case Month of
    1:  Result := '１月';
    2:  Result := '２月';
    3:  Result := '３月';
    4:  Result := '４月';
    5:  Result := '５月';
    6:  Result := '６月';
    7:  Result := '７月';
    8:  Result := '８月';
    9:  Result := '９月';
    10: Result := '10月';
    11: Result := '11月';
    12: Result := '12月';
    else Result := '';
    end;
  2:case Month of
    1:  Result := '睦月';
    2:  Result := '如月';
    3:  Result := '弥生';
    4:  Result := '卯月';
    5:  Result := '皐月';
    6:  Result := '水無月';
    7:  Result := '文月 ';
    8:  Result := '葉月';
    9:  Result := '長月';
    10: Result := '神無月';
    11: Result := '霜月';
    12: Result := '師走';
    else Result := '';
    end;
  else
    case Month of
    1:  Result := 'Jan';
    2:  Result := 'Feb';
    3:  Result := 'Mar';
    4:  Result := 'Apr';
    5:  Result := 'May';
    6:  Result := 'Jun';
    7:  Result := 'Jul';
    8:  Result := 'Aug';
    9:  Result := 'Sep';
    10: Result := 'Oct';
    11: Result := 'Nov';
    12: Result := 'Dec';
    else Result := '';
    end;
  end;
end;

function RectOfArea(R: TRect; ColCount, RowCount: Integer;
  Width, Height: Integer): TRect; overload; inline;
begin
  Result.Left := Width * R.Left div ColCount;
  Result.Right := Width * R.Right div ColCount;
  Result.Top := Height * R.Top div RowCount;
  Result.Bottom := Height * R.Bottom div RowCount;
end;

function RectOfArea(Col, Row: Integer; ColCount, RowCount: Integer;
  Width, Height: Integer): TRect; overload; inline;
var
  R: TRect;
begin
  R.Left := Col;
  R.Top := Row;
  R.Right := Col + 1;
  R.Bottom := Row + 1;
  Result := RectOfArea(R, ColCount, RowCount, Width, Height);
end;

{ TMyCalendar }

constructor TMyCalendar.Create(AYear: WORD);
begin
  SetYear(AYear);
end;

procedure TMyCalendar.SetYear(AValue: WORD);
var
  ST: TSystemTime;
begin
  if(AValue >= 1900)and(AValue <= 2099)then
    FYear := AValue
  else
  begin
    Windows.GetLocalTime(ST);
    if ST.Month < 4 then
      ST.Year := ST.Year - 1;
    FYear := ST.Year;
  end;
end;

procedure TMyCalendar.SetJpnFiscalYear(AValue: Boolean);
begin
  if FJpnFiscalYear <> AValue then
  begin
    FJpnFiscalYear := AValue;
  end;
end;

function TMyCalendar.MakeEMF(FileName: string): THandle;
var
  PrintWidth, PrintHeight: Integer;
  pwFileName: PWideChar;
  DC: HDC;
  R, R1, R2, R3: TRect;
  Canvas: TCanvas;
  S: string;
var
  IDate, DT1, DT2, DT3: Integer;
  wYear, wMonth, wDay, DOW: Word;
  DOW1, DOW2: WORD;
  DayOfYear, WeekOfYear: Integer;
var
  LeftColCount: Integer;
  MonthColCount: Integer;
  WeekColCount: Integer;
  WeekNumberColCount: Integer;
  CenterColCount: Integer;
  RightColCount: Integer;

  TopRowCount: Integer;
  TitleRowCount: Integer;
  BelowTitleRowCount: Integer;
  TotalRowCount: Integer;

  TextStyle: TTextStyle;
  function TotalColCount: Integer; //inline;
  begin
    Result := LeftColCount +
      (MonthColCount + WeekColCount + WeekNumberColCount) * 2 +
      CenterColCount +
      RightColCount;
  end;
  function LaterHalfColCount: Integer;
  begin
    Result := LeftColCount +
      (MonthColCount + WeekColCount + WeekNumberColCount) +
      CenterColCount;
  end;
  function HeaderRowCount: Integer;
  begin
    Result := TopRowCount;
    Inc(Result, TitleRowCount);
    Inc(Result, BelowTitleRowCount);
  end;
  function _RectOfArea(Col, Row: Integer): TRect; overload;
  begin
    Result := RectOfArea(Col, Row, TotalColCount, TotalRowCount, PrintWidth, PrintHeight);
  end;
  function _RectOfArea(Col, Row, ColWidth, RowWidth: Integer): TRect; overload;
  var
    R: TRect;
  begin
    R.Left := Col;
    R.Top := Row;
    R.Right := Col + ColWidth;
    R.Bottom := Row + RowWidth;
    Result := RectOfArea(R, TotalColCount, TotalRowCount, PrintWidth, PrintHeight);
  end;
  function _RectOfArea(R: TRect): TRect; overload;
  begin
    Result := RectOfArea(R, TotalColCount, TotalRowCount, PrintWidth, PrintHeight);
  end;
  procedure _TextRect(R: TRect; S: string);
  begin
    R.Top := -R.Top;
    R.Bottom := -R.Bottom;
    Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
  end;
  function _IntToStr_Knj(Value: Integer): string;
  var
    WS: WideString;
    I: Integer;
    procedure _Sub(S: string);
    begin
      Result := Result + S;
    end;
  begin
    Result := '';
    WS := UTF8Decode(IntToStr(Value));
    for I := 1 to Length(WS) do
    begin
      case WS[I] of
      '1': _Sub('一');
      '2': _Sub('二');
      '3': _Sub('三');
      '4': _Sub('四');
      '5': _Sub('五');
      '6': _Sub('六');
      '7': _Sub('七');
      '8': _Sub('八');
      '9': _Sub('九');
      '0': _Sub('〇');
      end;
    end;

  end;
begin
  if FileName = '' then
    pwFileName := nil
  else
    pwFileName := PWideChar(UTF8Decode(FileName));

  PrintWidth  := 210 * 100; // 0.01mm単位の幅
  PrintHeight := 297 * 100; // 0.01mm単位の高さ

  R := Classes.Rect(0, 0, PrintWidth, PrintHeight);
  DC := CreateEnhMetaFileW(0, pwFileName, @R, nil);
  try
    Windows.SetMapMode(DC, MM_HIMETRIC); // 0.01mm単位に設定
    Canvas := TCanvas.Create;
    try
      Canvas.Handle := DC;
      Canvas.Brush.Style := bsClear;

      LeftColCount := 1;
      MonthColCount := 1;
      WeekColCount := 7;
      WeekNumberColCount := 1;
      CenterColCount := 1;
      RightColCount := 1;

      TopRowCount := 1;
      TitleRowCount := 1;
      BelowTitleRowCount := 1;
      TotalRowCount := HeaderRowCount + 27 + 2;

      if FJpnFiscalYear then
      begin
        DT1 := Round(EncodeDate(FYear, 4, 1));
        DT2 := Round(EncodeDate(FYear, 10, 1));
        DT3 := Round(EncodeDate(FYear + 1, 4, 1));
      end
      else
      begin
        DT1 := Round(EncodeDate(FYear, 1, 1));
        DT2 := Round(EncodeDate(FYear, 7, 1));
        DT3 := Round(EncodeDate(FYear + 1, 1, 1));
      end;

      // タイトル（年度表示）
      begin
        R := _RectOfArea(
            0, TopRowCount, TotalColCount, TitleRowCount);

        if FJpnFiscalYear then
        begin
          if FLang = 1 then
            S := IntToStr(FYear) + '年度'
          else if FLang = 2 then
            S := _IntToStr_Knj(FYear) + '年度'
          else
            S := 'FY' + IntToStr(FYear);
        end
        else
        begin
          if FLang = 1 then
            S := IntToStr(FYear) + '年'
          else if FLang = 2 then
            S := _IntToStr_Knj(FYear) + '年'
          else
            S := {'FY' +} IntToStr(FYear);
        end;
        Canvas.Font.Height := R.Bottom - R.Top;

        FillChar(TextStyle, sizeof(TextStyle), 0);
        TextStyle.Alignment := taCenter;
        TextStyle.Clipping := FALSE;
        TextStyle.EndEllipsis := FALSE;
        TextStyle.ExpandTabs := TRUE;
        TextStyle.Layout := tlCenter;
        TextStyle.Opaque := TRUE;
        TextStyle.RightToLeft := FALSE;
        TextStyle.ShowPrefix := TRUE;
        TextStyle.SingleLine := TRUE;
        TextStyle.SystemFont := FALSE;
        TextStyle.Wordbreak := FALSE;
        //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
        _TextRect(R, S);

      end;

      DecodeDateFully(DT1, wYear, wMonth, wDay, DOW1);
      DecodeDateFully(DT2, wYear, wMonth, wDay, DOW2);

      // 左側（１～６月）
      for IDate := DT1 to DT2 - 1 do
      begin
        DayOfYear := IDate - DT1 + 1;
        WeekOfYear := (DayOfYear + DOW1 + 5) div 7;

        DecodeDateFully(IDate, wYear, wMonth, wDay, DOW);

        R := _RectOfArea(
          LeftColCount + MonthColCount + (DOW - 1),
          HeaderRowCount + (WeekOfYear - 1));

        S := IntToStr(wDay);
        Canvas.Font.Height := (R.Bottom - R.Top) div 2;
        case DOW of
        1: Canvas.Font.Color := clRed; // 日曜日
        7: Canvas.Font.Color := clBlue; // 土曜日
        else Canvas.Font.Color := clBlack;
        end;

        if IsHoliday(IDate) then
          Canvas.Font.Color := clRed;

        TextStyle.Alignment := taCenter;
        TextStyle.Clipping := FALSE;
        TextStyle.EndEllipsis := FALSE;
        TextStyle.ExpandTabs := TRUE;
        TextStyle.Layout := tlCenter;
        TextStyle.Opaque := TRUE;
        TextStyle.RightToLeft := FALSE;
        TextStyle.ShowPrefix := TRUE;
        TextStyle.SingleLine := TRUE;
        TextStyle.SystemFont := FALSE;
        TextStyle.Wordbreak := FALSE;
        //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
        _TextRect(R, S);

        if (wDay = 1)then
        begin
          // 「月」名の表示
          if DOW = 1 then
            R := _RectOfArea(
              LeftColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear - 1)
          else
            R := _RectOfArea(
              LeftColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear + 1 - 1);

          S := GetShortMonthName(wMonth, FLang);
          Canvas.Font.Height := (R.Bottom - R.Top) div 2;
          Canvas.Font.Color := clBlack;

          TextStyle.Alignment := taCenter;
          TextStyle.Clipping := FALSE;
          TextStyle.EndEllipsis := FALSE;
          TextStyle.ExpandTabs := TRUE;
          TextStyle.Layout := tlCenter;
          TextStyle.Opaque := TRUE;
          TextStyle.RightToLeft := FALSE;
          TextStyle.ShowPrefix := TRUE;
          TextStyle.SingleLine := TRUE;
          TextStyle.SystemFont := FALSE;
          TextStyle.Wordbreak := FALSE;
          //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
          _TextRect(R, S);


          // 「月」の境界線を引く
          if((wMonth <> 4)and(FJpnFiscalYear = TRUE)) or
            ((wMonth <> 1)and(FJpnFiscalYear = FALSE)) then
          begin
            R1 := _RectOfArea(
              LeftColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear - 1);
            R2 := _RectOfArea(
              LeftColCount + MonthColCount + DOW - 1,
              HeaderRowCount + WeekOfYear - 1);
            R3 := _RectOfArea(
                LaterHalfColCount - WeekNumberColCount - CenterColCount,
                HeaderRowCount + WeekOfYear - 1,
                //WeekNumberColCount,
                WeekNumberColCount - 1,
                1);

            Canvas.Pen.Style := psDot;
            Canvas.Pen.Color := clBlack;
            if DOW = 1 then
            begin
              Canvas.MoveTo(R1.Left, -R1.Top);
              Canvas.LineTo(R2.Left, -R2.Top);
            end
            else
            begin
              Canvas.MoveTo(R1.Left, -R1.Bottom);
              Canvas.LineTo(R2.Left, -R2.Bottom);
              Canvas.LineTo(R2.Left, -R2.Top);
            end;
            Canvas.LineTo(R3.Right, -R2.Top);
          end;
        end;// for I := DT1 to DT2 - 1 do // 左側（１～６月）


        // 「週カウント」の表示
        if FWeekNum then
        if DOW = 7 then
        begin
          R := _RectOfArea(
            LeftColCount + MonthColCount + WeekColCount,
            HeaderRowCount + WeekOfYear - 1);

          DayOfYear := IDate - DT1 + 1;
          WeekOfYear := (DayOfYear + DOW1 + 5) div 7;
          S := '(' + IntToStr(WeekOfYear) + ')';
          Canvas.Font.Height := (R.Bottom - R.Top) div 3;
          Canvas.Font.Color := clGray;

          TextStyle.Alignment := taCenter;
          TextStyle.Clipping := FALSE;
          TextStyle.EndEllipsis := FALSE;
          TextStyle.ExpandTabs := TRUE;
          TextStyle.Layout := tlCenter;
          TextStyle.Opaque := TRUE;
          TextStyle.RightToLeft := FALSE;
          TextStyle.ShowPrefix := TRUE;
          TextStyle.SingleLine := TRUE;
          TextStyle.SystemFont := FALSE;
          TextStyle.Wordbreak := FALSE;
          //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
          _TextRect(R, S);
        end;
      end;// for I := DT1 to DT2 - 1 do // 左側（１～６月）

      // 右側（７～１２月）
      for IDate := DT2 to DT3 - 1 do
      begin
        DayOfYear := IDate - DT2 + 1;
        WeekOfYear := (DayOfYear + DOW2 + 5) div 7;

        DecodeDateFully(IDate, wYear, wMonth, wDay, DOW);

        R := _RectOfArea(
          LaterHalfColCount + MonthColCount + DOW - 1,
          HeaderRowCount + WeekOfYear - 1);

        S := IntToStr(wDay);
        Canvas.Font.Height := (R.Bottom - R.Top) div 2;
        case DOW of
        1: Canvas.Font.Color := clRed; // 日曜日
        7: Canvas.Font.Color := clBlue; // 土曜日
        else Canvas.Font.Color := clBlack;
        end;

        if IsHoliday(IDate) then
          Canvas.Font.Color := clRed;

        TextStyle.Alignment := taCenter;
        TextStyle.Clipping := FALSE;
        TextStyle.EndEllipsis := FALSE;
        TextStyle.ExpandTabs := TRUE;
        TextStyle.Layout := tlCenter;
        TextStyle.Opaque := TRUE;
        TextStyle.RightToLeft := FALSE;
        TextStyle.ShowPrefix := TRUE;
        TextStyle.SingleLine := TRUE;
        TextStyle.SystemFont := FALSE;
        TextStyle.Wordbreak := FALSE;
        //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
        _TextRect(R, S);

        if (wDay = 1) then
        begin
          // 「月」名の表示
          if DOW = 1 then
            R := _RectOfArea(
              LaterHalfColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear - 1)
          else
            R := _RectOfArea(
              LaterHalfColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear + 1 - 1);

          S := GetShortMonthName(wMonth, FLang);
          Canvas.Font.Height := (R.Bottom - R.Top) div 2;
          Canvas.Font.Color := clBlack;

          TextStyle.Alignment := taCenter;
          TextStyle.Clipping := FALSE;
          TextStyle.EndEllipsis := FALSE;
          TextStyle.ExpandTabs := TRUE;
          TextStyle.Layout := tlCenter;
          TextStyle.Opaque := TRUE;
          TextStyle.RightToLeft := FALSE;
          TextStyle.ShowPrefix := TRUE;
          TextStyle.SingleLine := TRUE;
          TextStyle.SystemFont := FALSE;
          TextStyle.Wordbreak := FALSE;
          //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
          _TextRect(R, S);

          // 「月」の境界線を引く
          if((wMonth <> 10)and(FJpnFiscalYear = TRUE)) or
            ((wMonth <> 7)and(FJpnFiscalYear = FALSE)) then
          begin
            R1 := _RectOfArea(
              LaterHalfColCount + MonthColCount + (0) - 1,
              HeaderRowCount + WeekOfYear - 1);
            R2 := _RectOfArea(
              LaterHalfColCount + MonthColCount + DOW - 1,
              HeaderRowCount + WeekOfYear - 1);
            R3 := _RectOfArea(
                TotalColCount - WeekNumberColCount - RightColCount,
                HeaderRowCount + WeekOfYear - 1,
                //WeekNumberColCount,
                WeekNumberColCount - 1,
                1);
            Canvas.Pen.Style := psDot;
            Canvas.Pen.Color := clBlack;
            if DOW = 1 then
            begin
              Canvas.MoveTo(R1.Left, -R1.Top);
              Canvas.LineTo(R2.Left, -R2.Top);
            end
            else
            begin
              Canvas.MoveTo(R1.Left, -R1.Bottom);
              Canvas.LineTo(R2.Left, -R2.Bottom);
              Canvas.LineTo(R2.Left, -R2.Top);
            end;
            Canvas.LineTo(R3.Right, -R2.Top);
          end;
        end;

        // 「週カウント」の表示
        if FWeekNum then
        if DOW = 7 then
        begin
          R := _RectOfArea(
            LaterHalfColCount + MonthColCount + WeekColCount,
            HeaderRowCount + WeekOfYear - 1);

          DayOfYear := IDate - DT1 + 1;
          WeekOfYear := (DayOfYear + DOW1 + 5) div 7;
          S := '(' + IntToStr(WeekOfYear) + ')';
          Canvas.Font.Height := (R.Bottom - R.Top) div 3;
          Canvas.Font.Color := clGray;

          TextStyle.Alignment := taCenter;
          TextStyle.Clipping := FALSE;
          TextStyle.EndEllipsis := FALSE;
          TextStyle.ExpandTabs := TRUE;
          TextStyle.Layout := tlCenter;
          TextStyle.Opaque := TRUE;
          TextStyle.RightToLeft := FALSE;
          TextStyle.ShowPrefix := TRUE;
          TextStyle.SingleLine := TRUE;
          TextStyle.SystemFont := FALSE;
          TextStyle.Wordbreak := FALSE;
          //Canvas.TextRect(R, R.Left, R.Top, S, TextStyle);
          _TextRect(R, S);
        end;
      end;// for I := DT2 to DT3 - 1 do // 右側（７～１２月）

      Canvas.Handle := 0;
    finally
      Canvas.Free;
    end;
    Result := Windows.CloseEnhMetaFile(DC);
  finally
    Windows.DeleteDC(DC);
  end;
end;

procedure TMyCalendar.MakeEMFFile(FileName: string);
var
  EMF: THandle;
begin
  LazUTF8Classes.TFileStreamUTF8.Create(FileName, fmCreate).Free;
  EMF := MakeEMF(FileName);
  Windows.DeleteEnhMetaFile(EMF);
end;

initialization
  Holidays := TStringList.Create;
  Holidays.AddObject('元旦',		TObject( 101));		//  1月 1日
  Holidays.AddObject('建国記念の日',	TObject( 211));		//  2月11日
  Holidays.AddObject('昭和の日',	TObject( 429));		//  4月29日
  Holidays.AddObject('憲法記念日',	TObject( 503));		//  5月 3日
  Holidays.AddObject('みどりの日',	TObject( 504));		//  5月 4日
  Holidays.AddObject('こどもの日',	TObject( 505));		//  5月 5日
  Holidays.AddObject('山の日',		TObject( 811));		//  8月11日
  Holidays.AddObject('文化の日',	TObject(1103));		// 11月 3日
  Holidays.AddObject('勤労感謝の日',	TObject(1123));		// 11月23日
  Holidays.AddObject('天皇誕生日',	TObject(1223));		// 12月23日
  Holidays.AddObject('成人の日',	TObject( 121 + 40));	//  1月第2月曜
  Holidays.AddObject('海の日',		TObject( 731 + 40));	//  7月第3月曜
  Holidays.AddObject('敬老の日',	TObject( 931 + 40));	//  9月第3月曜
  Holidays.AddObject('体育の日',	TObject(1021 + 40));	// 10月第2月曜

  Holidays.AddObject('海の日',		TObject(2020 * 10000 + 723));	// 2020年は7月23日
  Holidays.AddObject('体育の日',	TObject(2020 * 10000 + 724));	// 2020年は7月24日
  Holidays.AddObject('山の日',		TObject(2020 * 10000 + 810));	// 2020年は8月10日

finalization
  Holidays.Free;
end.

