unit CardUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Controls, Contnrs, Graphics, ExtCtrls, Windows, LazUTF8Classes;

type

  { TMyCardItem }

  TMyCardItem = class
  private
    FItemName: string;
    FLeft, FTop, FWidth, FHeight: Integer;
    FOnClick: TNotifyEvent;
    FOnDblClick: TNotifyEvent;
    FOnInvalidate: TNotifyEvent;
    FOnMouseDown: TMouseEvent;
    FOnMouseMove: TMouseMoveEvent;
    FOnMouseUp: TMouseEvent;
    function GetBoundsRect: Classes.TRect;
    procedure SetBoundsRect(AValue: Classes.TRect);
  protected
    procedure DoPaint(ACanvas: TCanvas); virtual;
  public
    constructor Create(); virtual;
    destructor Destroy; override;
    procedure Invalidate;
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
    property ItemName: string read FItemName write FItemName;
    property Left: Integer read FLeft;
    property Top: Integer read FTop;
    property Width: Integer read FWidth;
    property Height: Integer read FHeight;
    property BoundsRect: Classes.TRect read GetBoundsRect write SetBoundsRect;
    property OnMouseDown: TMouseEvent read FOnMouseDown write FOnMouseDown;
    property OnMouseMove: TMouseMoveEvent read FOnMouseMove write FOnMouseMove;
    property OnMouseUp: TMouseEvent read FOnMouseUp write FOnMouseUp;
    property OnClick: TNotifyEvent read FOnClick write FOnClick;
    property OnDblClick: TNotifyEvent read FOnDblClick write FOnDblClick;
    property OnInvalidate: TNotifyEvent read FOnInvalidate write FOnInvalidate;
  end;

  { TMyCardLabel }

  TMyCardLabel = class(TMyCardItem)
  private
    FAlignment: TAlignment;
    FCaption: string;
    FLayout: TTextLayout;
    FFont: TFont;
    procedure SetAlignment(AValue: TAlignment);
    procedure SetCaption(AValue: string);
    procedure SetFont(AValue: TFont);
    procedure SetLayout(AValue: TTextLayout);
  protected
    procedure DoPaint(ACanvas: TCanvas); override;
  public
    constructor Create(); override;
    destructor Destroy; override;
    property Alignment: TAlignment read FAlignment write SetAlignment;
    property Caption: string read FCaption write SetCaption;
    property Font: TFont read FFont write SetFont;
    property Layout: TTextLayout read FLayout write SetLayout;
  end;

  { TMyCardImage }

  TMyCardImage = class(TMyCardItem)
  private
    FPicture: TPicture;
    procedure SetPicture(AValue: TPicture);
  protected
    procedure PictureChanged(Sender : TObject);
    procedure DoPaint(ACanvas: TCanvas); override;
  public
    constructor Create(); override;
    destructor Destroy; override;
    property Picture: TPicture read FPicture write SetPicture;
  end;

  { TMyCardShape }

  TMyCardShape = class(TMyCardItem)
  private
    FShape: Integer;
    FPen: TPen;
    FBrush: TBrush;
    procedure SetBrush(AValue: TBrush);
    procedure SetPen(AValue: TPen);
    procedure SetShape(AValue: Integer);
    procedure DoPaint_copied_from_Lazarus(ACanvas: TCanvas);
  protected
    procedure DoPaint(ACanvas: TCanvas); override;
  public
    constructor Create(); override;
    destructor Destroy; override;
    property Shape: Integer read FShape write SetShape default Integer(stRectangle);
    property Pen: TPen read FPen write SetPen;
    property Brush: TBrush read FBrush write SetBrush;
  end;

  { TMyCard }

  TMyCard = class(TObjectList)
  private
    FLeftMargin, FTopMargin: Integer;
    FColCount, FRowCount: Integer;
    FSelectedItem: TMyCardItem;
    function GetItems(Index: Integer): TMyCardItem;
    function GetSelectedItem: TMyCardItem;
    procedure SetSelectedItem(AValue: TMyCardItem);
  public
    constructor Create;
    destructor Destroy; override;
    procedure DoPrint_SingleCard(DC: HDC);
    procedure DoPrint(DC: HDC; OffsetX, OffsetY: Integer{; APanel: TPanel});
    function MakeEMF(FileName: string = ''): THandle;
    procedure MakeEMFFile(FileName: string);
    procedure BringToFront(Item: TMyCardItem);
    procedure SendToBack(Item: TMyCardItem);
  public
    property SelectedItem: TMyCardItem read GetSelectedItem write SetSelectedItem;
    property Items[Index: Integer]: TMyCardItem read GetItems; default;
  end;

implementation

procedure DrawTetraStar(Canvas: TCanvas; RDraw: TRect; Mode: Integer);
var
  R: TRect;
  I: Integer;
  PCross: array[0..7] of TPoint;
begin
  R := RDraw;
  if(R.Right - R.Left) < (R.Bottom - R.Top)then
  begin
    I := R.Right - R.Left;
    R.Top := (R.Top + R.Bottom - I) div 2;
    R.Bottom := R.Top + I;
  end
  else
  begin
    I := R.Bottom - R.Top;
    R.Left := (R.Left + R.Right - I) div 2;
    R.Right := R.Left + I;
  end;

  case Mode mod 4 of
  0:begin // 星型（十字）
      PCross[0] := Classes.Point((R.Left + R.Right) div 2, R.Top);
      PCross[1] := Classes.Point((R.Left * 5 + R.Right * 8) div 13, (R.Top * 8 + R.Bottom * 5) div 13);
      PCross[2] := Classes.Point(R.Right, (R.Top + R.Bottom) div 2);
      PCross[3] := Classes.Point((R.Left * 5 + R.Right * 8) div 13, (R.Top * 5 + R.Bottom * 8) div 13);
      PCross[4] := Classes.Point((R.Left + R.Right) div 2, R.Bottom);
      PCross[5] := Classes.Point((R.Left * 8 + R.Right * 5) div 13, (R.Top * 5 + R.Bottom * 8) div 13);
      PCross[6] := Classes.Point(R.Left, (R.Top + R.Bottom) div 2);
      PCross[7] := Classes.Point((R.Left * 8 + R.Right * 5) div 13, (R.Top * 8 + R.Bottom * 5) div 13);
      Canvas.Polygon(PCross);
    end;
  1:begin // 星型（×字）
      PCross[0] := Classes.Point(R.Left, R.Top);
      PCross[1] := Classes.Point((R.Left + R.Right) div 2, (R.Top * 7 + R.Bottom * 3) div 10);
      PCross[2] := Classes.Point(R.Right, R.Top);
      PCross[3] := Classes.Point((R.Left * 3 + R.Right * 7) div 10, (R.Top + R.Bottom) div 2);
      PCross[4] := Classes.Point(R.Right, R.Bottom);
      PCross[5] := Classes.Point((R.Left + R.Right) div 2, (R.Top * 3 + R.Bottom * 7) div 10);
      PCross[6] := Classes.Point(R.Left, R.Bottom);
      PCross[7] := Classes.Point((R.Left * 7 + R.Right * 3) div 10, (R.Top + R.Bottom) div 2);
      Canvas.Polygon(PCross);
    end;
  2:begin // 星型（十字）やや太め？
      PCross[0] := Classes.Point((R.Left + R.Right) div 2, R.Top);
      PCross[1] := Classes.Point((R.Left * 1 + R.Right * 2) div 3, (R.Top * 2 + R.Bottom * 1) div 3);
      PCross[2] := Classes.Point(R.Right, (R.Top + R.Bottom) div 2);
      PCross[3] := Classes.Point((R.Left * 1 + R.Right * 2) div 3, (R.Top * 1 + R.Bottom * 2) div 3);
      PCross[4] := Classes.Point((R.Left + R.Right) div 2, R.Bottom);
      PCross[5] := Classes.Point((R.Left * 2 + R.Right * 1) div 3, (R.Top * 1 + R.Bottom * 2) div 3);
      PCross[6] := Classes.Point(R.Left, (R.Top + R.Bottom) div 2);
      PCross[7] := Classes.Point((R.Left * 2 + R.Right * 1) div 3, (R.Top * 2 + R.Bottom * 1) div 3);
      Canvas.Polygon(PCross);
    end;
  3:begin // 星型（×字）やや太め？
      PCross[0] := Classes.Point(R.Left, R.Top);
      PCross[1] := Classes.Point((R.Left + R.Right) div 2, (R.Top * 4 + R.Bottom) div 5);
      PCross[2] := Classes.Point(R.Right, R.Top);
      PCross[3] := Classes.Point((R.Left + R.Right * 4) div 5, (R.Top + R.Bottom) div 2);
      PCross[4] := Classes.Point(R.Right, R.Bottom);
      PCross[5] := Classes.Point((R.Left + R.Right) div 2, (R.Top + R.Bottom * 4) div 5);
      PCross[6] := Classes.Point(R.Left, R.Bottom);
      PCross[7] := Classes.Point((R.Left * 4 + R.Right) div 5, (R.Top + R.Bottom) div 2);
      Canvas.Polygon(PCross);
    end;
  end;// case Mode mod 4 of
end;

procedure DrawTriStar(Canvas: TCanvas; RDraw: TRect; Mode: Integer);
var
  R: TRect;
  P: array[0..5] of TPoint;
  I: Integer;
  function _Proc(pA, pB, pC: TPoint): TPoint; inline;
  var
    pM: TPoint;
  const
    COEF = 9;
  begin
    pM := Classes.Point((pA.x + pB.x) div 2, (pA.y + pB.y) div 2);
    Result.x := (pM.x * COEF + pC.x * 2) div (COEF + 2);
    Result.y := (pM.y * COEF + pC.y * 2) div (COEF + 2);
  end;
  procedure _Draw; inline;
  begin
    P[1] := _Proc(P[0], P[2], P[4]);
    P[3] := _Proc(P[2], P[4], P[0]);
    P[5] := _Proc(P[4], P[0], P[2]);
    Canvas.Polygon(P);
  end;
begin
  R := RDraw;
  case Mode mod 4 of
  0, 2:
    if((R.Right - R.Left) * 13) < ((R.Bottom - R.Top) * 15)then
    begin
      I := (R.Right - R.Left) * 13 div 15;
      R.Top := (R.Top + R.Bottom - I) div 2;
      R.Bottom := R.Top + I;
    end
    else
    begin
      I := (R.Bottom - R.Top) * 15 div 13;
      R.Left := (R.Left + R.Right - I) div 2;
      R.Right := R.Left + I;
    end;
  else// 1, 3:
    if((R.Right - R.Left) * 15) < ((R.Bottom - R.Top) * 13)then
    begin
      I := (R.Right - R.Left) * 15 div 13;
      R.Top := (R.Top + R.Bottom - I) div 2;
      R.Bottom := R.Top + I;
    end
    else
    begin
      I := (R.Bottom - R.Top) * 13 div 15;
      R.Left := (R.Left + R.Right - I) div 2;
      R.Right := R.Left + I;
    end;
  end;

  case Mode mod 4 of
  0:begin
      P[0] := Classes.Point((R.Left + R.Right) div 2, R.Top);
      P[2] := Classes.Point(R.Right, R.Bottom);
      P[4] := Classes.Point(R.Left, R.Bottom);
    end;
  1:begin
      P[0] := Classes.Point(R.Right, (R.Top + R.Bottom) div 2);
      P[2] := Classes.Point(R.Left, R.Bottom);
      P[4] := Classes.Point(R.Left, R.Top);
    end;
  2:begin
      P[0] := Classes.Point((R.Left + R.Right) div 2, R.Bottom);
      P[2] := Classes.Point(R.Right, R.Top);
      P[4] := Classes.Point(R.Left, R.Top);
    end;
  else // 3:
    begin
      P[0] := Classes.Point(R.Left, (R.Top + R.Bottom) div 2);
      P[2] := Classes.Point(R.Right, R.Bottom);
      P[4] := Classes.Point(R.Right, R.Top);
    end;
  end;// case Mode mod 4 of
  _Draw();
end;

procedure DrawHeart(Canvas: TCanvas; RDraw: TRect);
var
  R: TRect;
  I: Integer;
  P: array[0..18] of TPoint;
  function _P(X, Y: Extended): Classes.TPoint;
  begin
    Result.x := Round((R.Left + R.Right + (R.Right - R.Left) * X) / 2);
    Result.y := Round((R.Top + R.Bottom + (R.Bottom - R.Top) * Y) / 2);
  end;
begin
  R := RDraw;
  if(R.Right - R.Left) < (R.Bottom - R.Top)then
  begin
    I := R.Right - R.Left;
    R.Top := (R.Top + R.Bottom - I) div 2;
    R.Bottom := R.Top + I;
  end
  else
  begin
    I := R.Bottom - R.Top;
    R.Left := (R.Left + R.Right - I) div 2;
    R.Right := R.Left + I;
  end;

  P[0]  := _P(0, 1);
  P[1]  := _P(-0.10, 0.7);

  P[2]  := _P(-1,  0.15);
  P[3]  := _P(-1, -0.25);
  P[4]  := _P(-1, -0.5);

  P[5]  := _P(-0.95, -1);
  P[6]  := _P(-0.50, -1);
  P[7]  := _P(-0.25, -1);

  P[8]  := _P(-0.08, -0.75);
  P[9]  := _P( 0.00, -0.60);
  P[10] := _P( 0.08, -0.75);

  P[11] := _P( 0.25, -1);
  P[12] := _P( 0.50, -1);
  P[13] := _P( 0.95, -1);

  P[14] := _P(1, -0.5);
  P[15] := _P(1, -0.25);
  P[16] := _P(1,  0.15);

  P[17] := _P(0.10, 0.7);
  P[18] := _P(0, 1);

  Canvas.PolyBezier(P, TRUE, TRUE);
end;

{ TMyCardShape }

procedure TMyCardShape.SetShape(AValue: Integer);
begin
  if FShape = AValue then Exit;
  FShape := AValue;
  Invalidate;
end;

procedure TMyCardShape.SetBrush(AValue: TBrush);
begin
  FBrush.Assign(AValue);
  Invalidate;
end;

procedure TMyCardShape.SetPen(AValue: TPen);
begin
  FPen.Assign(AValue);
  Invalidate;
end;

procedure TMyCardShape.DoPaint(ACanvas: TCanvas);
begin
  inherited DoPaint(ACanvas);

  ACanvas.Pen := FPen;
  ACanvas.Pen.Width := FPen.Width * 16;
  ACanvas.Brush := FBrush;
  ACanvas.Brush.Style := FBrush.Style;

  case FShape of
  100:// 横線
    ACanvas.Line(0, Height div 2, Width, Height div 2);
  101:// 縦線
    ACanvas.Line(Width div 2, 0, Width div 2, Height);
  102..105:// 四角星
    DrawTetraStar(ACanvas, Classes.Rect(0, 0, Width, Height), FShape - 102);
  106..109:// 三角星
    DrawTriStar(ACanvas, Classes.Rect(0, 0, Width, Height), FShape - 106);
  110:// ハート
    DrawHeart(ACanvas, Classes.Rect(0, 0, Width, Height));
  else DoPaint_copied_from_Lazarus(ACanvas);
  end;
end;

procedure TMyCardShape.DoPaint_copied_from_Lazarus(ACanvas: TCanvas);
var
  _Shape: TShapeType;
  PaintRect: TRect;
  MinSize: Longint;
  P: array[0..3] of TPoint;
  PStar: array[0..10] of TPoint;
  PenInc, PenDec: Integer;
  RadiusBig, RadiusBig2, RadiusSm, i: Integer;
  PCenter: TPoint;
  Alfa: Double;
  function ifthen(val:boolean;const iftrue:double ; const iffalse:double =0.0):double;  inline;
  begin
    if val then result:=iftrue else result:=iffalse;
  end;
  function GetStarAngle(N: Integer; ADown: boolean): Double;
  begin
    Result := pi/5 * N + pi/2 * IfThen(ADown, -1, 1);
  end;
begin
  _Shape := TShapeType(FShape);
  with ACanvas do
  begin
    PenInc := Pen.Width div 2;
    PenDec := (Pen.Width - 1) div 2;

    PaintRect := Classes.Rect(PenInc, PenInc, Self.Width - PenDec, Self.Height - PenDec);
    if PaintRect.Left = PaintRect.Right then
      PaintRect.Right := PaintRect.Right + 1;
    if PaintRect.Top = PaintRect.Bottom then
      PaintRect.Bottom := PaintRect.Bottom + 1;

    MinSize := Min(PaintRect.Right - PaintRect.Left, PaintRect.Bottom - PaintRect.Top);
    if _Shape in [stSquare, stRoundSquare, stCircle, stSquaredDiamond] then
    begin
      PaintRect.Left := PaintRect.Left + ((PaintRect.Right - PaintRect.Left) - MinSize) div 2;
      PaintRect.Top := PaintRect.Top + ((PaintRect.Bottom - PaintRect.Top) - MinSize) div 2;
      PaintRect.Right := PaintRect.Left + MinSize;
      PaintRect.Bottom := PaintRect.Top + MinSize;
    end;

    case _Shape of
      stRectangle, stSquare:
        Rectangle(PaintRect);
      stRoundRect, stRoundSquare:
        RoundRect(PaintRect, MinSize div 4, MinSize div 4);
      stCircle, stEllipse:
        Ellipse(PaintRect);
      stSquaredDiamond, stDiamond:
        begin
          P[0].x := PaintRect.Left;
          P[0].y := (PaintRect.Top + PaintRect.Bottom) div 2;
          P[1].x := (PaintRect.Left + PaintRect.Right) div 2;
          P[1].y := PaintRect.Top;
          P[2].x := PaintRect.Right - 1;
          P[2].y := P[0].y;
          P[3].x := P[1].x;
          P[3].y := PaintRect.Bottom - 1;
          Polygon(P);
        end;
      stTriangle:
        begin
          with Self do
          begin
            P[0].x := (Width - 1) div 2;
            P[0].y := PenInc;
            P[1].x := Width - PenInc - 1;
            P[1].y := Height - PenInc - 1;
            P[2].x := PenInc;
            P[2].y := Height - PenInc - 1;
            P[3].x := P[0].x;
            P[3].y := P[0].y;
            Polygon(P);
          end;
        end;
      stTriangleDown:
        begin
          with Self do
          begin
            P[0].x := (Width - 1) div 2;
            P[0].y := Height - PenInc - 1;
            P[1].x := Width - PenInc - 1;
            P[1].y := PenInc;
            P[2].x := PenInc;
            P[2].y := PenInc;
            P[3].x := P[0].x;
            P[3].y := P[0].y;
            Polygon(P);
          end;
        end;
      stTriangleLeft:
        begin
          with Self do
          begin
            P[0].x := PenInc;
            P[0].y := Height div 2;
            P[1].x := Width - PenInc - 1;
            P[1].y := PenInc;
            P[2].x := Width - PenInc - 1;
            P[2].y := Height - PenInc - 1;
            P[3].x := P[0].x;
            P[3].y := P[0].y;
            Polygon(P);
          end;
        end;
      stTriangleRight:
        begin
          with Self do
          begin
            P[0].x := Width - PenInc - 1;
            P[0].y := Height div 2;
            P[1].x := PenInc;
            P[1].y := PenInc;
            P[2].x := PenInc;
            P[2].y := Height - PenInc - 1;
            P[3].x := P[0].x;
            P[3].y := P[0].y;
            Polygon(P);
          end;
        end;
      stStar,
      stStarDown:
        begin
          with Self do
          begin
            //radius if star scaled by height
            RadiusBig := Trunc((Height-Pen.Width) / (1+cos(pi/5)));
            //radius if star scaled by width
            RadiusBig2 := Trunc((Width-Pen.Width) / (2*sin(pi*2/5)));

            if RadiusBig<=RadiusBig2 then
            begin
              if _Shape = stStar then
                PCenter.Y := RadiusBig+PenDec
              else
                PCenter.Y := Height-RadiusBig-PenDec;
            end
            else
            begin
              RadiusBig := RadiusBig2;
              PCenter.Y := Height div 2;
            end;
            PCenter.X := Width div 2;
            RadiusSm := RadiusBig * 57 div 150;

            for i := 0 to 4 do
            begin
              Alfa := GetStarAngle(i*2, _Shape = stStarDown);
              PStar[i*2].x := PCenter.X + Round(RadiusBig*cos(Alfa));
              PStar[i*2].y := PCenter.Y - Round(RadiusBig*sin(Alfa));
              Alfa:= GetStarAngle(i*2+1, _Shape = stStarDown);
              PStar[i*2+1].x := PCenter.X + Round(RadiusSm*cos(Alfa));
              PStar[i*2+1].y := PCenter.Y - Round(RadiusSm*sin(Alfa));
            end;

            PStar[10] := PStar[0];
            Polygon(PStar);
          end;
        end;
    end;
  end;
end;

constructor TMyCardShape.Create();
begin
  inherited Create();
  FPen := TPen.Create;
  with FPen do
  begin
    Color := clBlack;
    Mode := pmCopy;
    Width := 1;
  end;
  FBrush := TBrush.Create;
  FBrush.Style := bsClear;
end;

destructor TMyCardShape.Destroy;
begin
  inherited Destroy;
end;

{ TMyCardItem }

constructor TMyCardItem.Create();
begin
  //inherited Create(AOwner);
end;

destructor TMyCardItem.Destroy;
begin
  inherited Destroy;
end;

procedure TMyCardItem.Invalidate;
begin
  if Assigned(FOnInvalidate) then
    FOnInvalidate(Self);
end;

procedure TMyCardItem.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  if(FLeft <> ALeft)and(FTop <> ATop)or(FWidth <> AWidth)or(FHeight <> AHeight)then
  begin
    FLeft := ALeft;
    FTop := ATop;
    FWidth := AWidth;
    FHeight := AHeight;
    Invalidate;
  end;
end;

function TMyCardItem.GetBoundsRect: Classes.TRect;
begin
  Result.Left   := FLeft;
  Result.Top    := FTop;
  Result.Right  := FLeft + FWidth;
  Result.Bottom := FTop  + FHeight;
end;

procedure TMyCardItem.SetBoundsRect(AValue: Classes.TRect);
var
  W, H: Integer;
begin
  W := AValue.Right - AValue.Left;
  H := AValue.Bottom - AValue.Top;
  if(FLeft <> AValue.Left)or(FTop <> AValue.Top)or(FWidth <> W)or(FHeight <> H)then
  begin
    FLeft := AValue.Left;
    FTop := AValue.Top;
    FWidth := W;
    FHeight := H;
    Invalidate;
  end;
end;

procedure TMyCardItem.DoPaint(ACanvas: TCanvas);
begin
  {
  if ACanvas <> Self.Canvas then
  begin
    ACanvas.Font := Self.Canvas.Font;
    ACanvas.Brush := Self.Canvas.Brush;
    ACanvas.Pen := Self.Canvas.Pen;
  end;
  }
end;

{ TMyCardLabel }

procedure TMyCardLabel.SetAlignment(AValue: TAlignment);
begin
  if FAlignment = AValue then Exit;
  FAlignment := AValue;
  Invalidate;
end;

procedure TMyCardLabel.SetCaption(AValue: string);
begin
  if FCaption = AValue then
    Exit;
  FCaption := AValue;
  Invalidate;
end;

procedure TMyCardLabel.SetFont(AValue: TFont);
begin
  FFont.Assign(AValue);
  Invalidate;
end;

procedure TMyCardLabel.SetLayout(AValue: TTextLayout);
begin
  if FLayout = AValue then Exit;
  FLayout := AValue;
  Invalidate;
end;

procedure TMyCardLabel.DoPaint(ACanvas: TCanvas);
var
  R: TRect;
  TS : TTextStyle;
  E: Extended;
begin
  inherited DoPaint(ACanvas);
  R := Classes.Rect(0, 0, Width, Height);
  FillChar(TS, SizeOf(TS), 0);
  with TS do
  begin
    Alignment := FAlignment;
    Layout := FLayout;
    //Clipping := TRUE;
  end;

  ACanvas.Font.Assign(FFont);
  E := -(FFont.Size * 2540 / 72);
  // 0.5ポイント単位で指定できるように２倍数で内部データを保持、
  // ここで元に戻す（記録値の２分の１）
  // でも、標準のフォント設定ダイアログは１ポイント単位の指定しか対応していないから、
  // う～ん、とりあえずダイアログ上では２倍値の整数を表示しているけれど．．．
  ACanvas.Font.Height := Round(E / 2);

  ACanvas.TextRect(R, R.Left, R.Top, Caption, TS);
end;

constructor TMyCardLabel.Create();
begin
  inherited Create();
  FFont := TFont.Create;
  FFont.Size := 12;
  FAlignment := taCenter;
  FLayout := tlCenter;
end;

destructor TMyCardLabel.Destroy;
begin
  FFont.Free;
  inherited Destroy;
end;

{ TMyCardImage }

procedure TMyCardImage.SetPicture(AValue: TPicture);
begin
  if FPicture = AValue then Exit;
  FPicture.Assign(AValue);
end;

procedure TMyCardImage.PictureChanged(Sender: TObject);
begin
  Invalidate;
end;

procedure TMyCardImage.DoPaint(ACanvas: TCanvas);
var
  R: TRect;
  I: Integer;
begin
  inherited DoPaint(ACanvas);
  if Assigned(FPicture.Graphic)and
    (FPicture.Graphic.Width > 0) and (FPicture.Graphic.Height > 0)then
  begin
    if(Width * FPicture.Graphic.Height) > (Height * FPicture.Graphic.Width)then
    begin
      R.Top := 0;
      R.Bottom := Height;
      I := Height * FPicture.Graphic.Width div FPicture.Graphic.Height;
      R.Left := (Width - I) div 2;
      R.Right := R.Left + I;
    end
    else
    begin
      R.Left := 0;
      R.Right := Width;
      I := Width * FPicture.Graphic.Height div FPicture.Graphic.Width;
      R.Top := (Height - I) div 2;
      R.Bottom := R.Top + I;
    end;
    ACanvas.StretchDraw(R, FPicture.Graphic);
  end;
end;

constructor TMyCardImage.Create();
begin
  inherited Create();
  FPicture := TPicture.Create;
  FPicture.OnChange := @PictureChanged;
end;

destructor TMyCardImage.Destroy;
begin
  FPicture.OnChange := nil;
  FPicture.Graphic := nil;
  FPicture.Free;
  inherited Destroy;
end;

{ TMyCard }

procedure TMyCard.DoPrint_SingleCard(DC: HDC);
var
  ACanvas: TCanvas;
  I, SaveIndex: Integer;
  MyCardItem: TMyCardItem;
begin
  // １枚分の描画を実行
  ACanvas := TCanvas.Create;
  try
    for I := 0 to Self.Count - 1 do
    begin
      MyCardItem := Items[I];
      SaveIndex := Windows.SaveDC(DC);
      try
        Windows.OffsetWindowOrgEx(DC, -MyCardItem.Left, -MyCardItem.Top, nil);
        ACanvas.Handle := DC;
        MyCardItem.DoPaint(ACanvas);
        ACanvas.Handle := 0;
      finally
        Windows.RestoreDC(DC, SaveIndex);
      end;
    end; // for I := 0 to APanel.ControlCount - 1 do
  finally
    ACanvas.Free;
  end;
end;

procedure TMyCard.DoPrint(DC: HDC; OffsetX, OffsetY: Integer{; APanel: TPanel});
var
  iHorzSize, iVertSize, iHorzRes, iVertRes: Integer;
  X, Y: Integer;
  PrintRect: TRect;
begin
  iHorzSize := GetDeviceCaps(DC, HORZSIZE); // mm単位の幅
  iVertSize := GetDeviceCaps(DC, VERTSIZE); // mm単位の高さ
  iHorzRes  := GetDeviceCaps(DC, HORZRES);  // ピクセル単位の幅
  iVertRes  := GetDeviceCaps(DC, VERTRES);  // ピクセル単位の高さ

  for Y := 0 to FRowCount - 1 do
  begin
    for X := 0 to FColCount - 1 do
    begin
      // １枚分の描画エリアをピクセル単位で計算
      PrintRect.Left   := (FLeftMargin + 91 * X) * iHorzRes div iHorzSize;
      PrintRect.Top    := (FTopMargin  + 55 * Y) * iVertRes div iVertSize;
      PrintRect.Right  := PrintRect.Left + 91 * iHorzRes div iHorzSize;
      PrintRect.Bottom := PrintRect.Top  + 55 * iVertRes div iVertSize;
      OffsetRect(PrintRect, OffsetX, OffsetY);

      // Panel上のコントロールによる描画範囲（Panel.ClientRect）が
      // 描画エリア（１枚分）に一致するように、位置を合わせる

      // 位置合わせができる描画モードに設定
      if Windows.SetMapMode(DC, MM_ANISOTROPIC) = 0 then
        // SetMapModeが失敗
      // 描画エリアの左上隅をピクセル単位で設定
      else if Windows.SetViewportOrgEx(DC, PrintRect.Left, PrintRect.Top, nil) = FALSE then
        // SetViewportOrgExが失敗
      // 描画エリアの高さと幅をピクセル単位で設定
      else if Windows.SetViewportExtEx(DC,
        PrintRect.Right - PrintRect.Left, PrintRect.Bottom - PrintRect.Top, nil) = FALSE then
        // SetViewportExtExが失敗
      // 描画元の高さと幅を設定　（0.01mm単位）
      else if Windows.SetWindowExtEx(DC, 91 * 100, 55 * 100, nil) = FALSE then
        // SetWindowsExtExが失敗
      else
        // １枚分の描画を実行
        DoPrint_SingleCard(DC);

      // 製品クラスの信頼性を得るには、上記の各関数が返すエラーもちゃんと拾いましょう。

    end;// for X := 0 to FColCount - 1 do
  end;// for Y := 0 to FRowCount - 1 do
end;

function TMyCard.MakeEMF(FileName: string): THandle;
var
  pwFileName: PWideChar;
  DC: HDC;
  R: TRect;
begin
  if FileName = '' then
    pwFileName := nil
  else
    pwFileName := PWideChar(UTF8Decode(FileName));

  // 幅210mm × 高さ297mm
  R := Classes.Rect(0, 0, 210 * 100, 297 * 100);
  DC := CreateEnhMetaFileW(0, pwFileName, @R, nil);
  try
    DoPrint(DC, 0, 0{, APanel});
    Result := Windows.CloseEnhMetaFile(DC);
  finally
    Windows.DeleteDC(DC);
  end;
end;

function TMyCard.GetSelectedItem: TMyCardItem;
begin
  if Self.IndexOf(FSelectedItem) >= 0 then
    Result := FSelectedItem
  else
    Result := nil;
end;

function TMyCard.GetItems(Index: Integer): TMyCardItem;
begin
  if(Index >= 0)and(Index < Count)then
    Result := TMyCardItem(inherited Items[Index])
  else
    Result := nil;
end;

procedure TMyCard.SetSelectedItem(AValue: TMyCardItem);
begin
  if FSelectedItem <> AValue then
  begin
    FSelectedItem := AValue;
  end;
end;

constructor TMyCard.Create;
begin
  inherited Create(TRUE);
  // 日本の名刺サイズ（91×55mm）2×5枚をA4紙（210×297mm）の中央に印刷する場合を想定
  FLeftMargin := 14; // 14mm
  FTopMargin  := 11; // 11mm
  FColCount   := 2; // 2列
  FRowCount   := 5; // 5段
end;

destructor TMyCard.Destroy;
begin
  inherited Destroy;
end;

procedure TMyCard.MakeEMFFile(FileName: string);
var
  EMF: THandle;
begin
  LazUTF8Classes.TFileStreamUTF8.Create(FileName, fmCreate).Free;
  EMF := MakeEMF(FileName);
  Windows.DeleteEnhMetaFile(EMF);
end;

procedure TMyCard.BringToFront(Item: TMyCardItem);
var
  I: Integer;
begin
  I := Self.IndexOf(Item);
  if I >= 0 then
    Self.Move(I, Self.Count - 1);
end;

procedure TMyCard.SendToBack(Item: TMyCardItem);
var
  I: Integer;
begin
  I := Self.IndexOf(Item);
  if I >= 0 then
    Self.Move(I, 0);
end;

end.

