unit LabelPropertyUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, Buttons, Spin, CheckLst, Windows;

type

  { TLabelPropertyForm }

  TLabelPropertyForm = class(TForm)
    Bevel_AlignmentLayout: TBevel;
    Bevel_Font: TBevel;
    Button_Ok: TButton;
    Button_Cancel: TButton;
    Button_Font: TButton;
    CheckListBox_FontStyle: TCheckListBox;
    ColorButton_FontColor: TColorButton;
    ComboBox_FontSize: TComboBox;
    ComboBox_FontName: TComboBox;
    FontDialog1: TFontDialog;
    Label_FontColor: TLabel;
    Label_AlignmentLayout: TLabel;
    Label_AlignmentLayout1: TLabel;
    Label_FontColor1: TLabel;
    Label_Preview: TLabel;
    LabeledEdit_Text: TLabeledEdit;
    SpeedButton10: TSpeedButton;
    SpeedButton11: TSpeedButton;
    SpeedButton12: TSpeedButton;
    SpeedButton13: TSpeedButton;
    SpeedButton14: TSpeedButton;
    SpeedButton15: TSpeedButton;
    SpeedButton16: TSpeedButton;
    SpeedButton17: TSpeedButton;
    SpeedButton18: TSpeedButton;
    procedure Button_FontClick(Sender: TObject);
    procedure CheckListBox_FontStyleClickCheck(Sender: TObject);
    procedure ColorButton_FontColorColorChanged(Sender: TObject);
    procedure ComboBox_FontNameChange(Sender: TObject);
    procedure ComboBox_FontSizeChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure LabeledEdit_TextChange(Sender: TObject);
    procedure SpeedButtonClick(Sender: TObject);
  private
    FFont: TFont;
    FOnPropertyChange: TNotifyEvent;
    function Get_Tag_of_Button_in_AlignmentLayout: Integer;
    function GetText: string;
    function GetFont: TFont;
    function GetAlignment: TAlignment;
    function GetLayout: TTextLayout;
    procedure Set_Tag_of_Button_in_AlignmentLayout(AValue: Integer);
    procedure SetText(AValue: string);
    procedure SetFont(AValue: TFont);
    procedure SetAlignment(AValue: TAlignment);
    procedure SetLayout(AValue: TTextLayout);
    { private declarations }
  public
    { public declarations }
    property Text: string read GetText write SetText;
    property Font: TFont read GetFont write SetFont;
    property Alignment: TAlignment read GetAlignment write SetAlignment;
    property Layout: TTextLayout read GetLayout write SetLayout;
    property OnPropertyChange: TNotifyEvent read FOnPropertyChange write FOnPropertyChange;
  end;

var
  LabelPropertyForm: TLabelPropertyForm;

implementation

{$R *.lfm}

{ TLabelPropertyForm }

procedure TLabelPropertyForm.LabeledEdit_TextChange(Sender: TObject);
begin
  Label_Preview.Caption := LabeledEdit_Text.Text;
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TLabelPropertyForm.SpeedButtonClick(Sender: TObject);
var
  I: Integer;
begin
  if Sender is TSpeedButton then
  begin
    Label_AlignmentLayout.Caption := Label_AlignmentLayout.Hint + ': ' + TSpeedButton(Sender).Hint;
    I := TSpeedButton(Sender).Tag;
    Label_Preview.Alignment := TAlignment(I mod 3);
    Label_Preview.Layout := TTextLayout((I div 3) mod 3);
    if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
  end;
end;

function TLabelPropertyForm.GetLayout: TTextLayout;
begin
  Result := TTextLayout((Get_Tag_of_Button_in_AlignmentLayout div 3)mod 3);
end;

function TLabelPropertyForm.Get_Tag_of_Button_in_AlignmentLayout: Integer;
var
  I: Integer;
  SpeedButton: TSpeedButton;
begin
  Result := 5;
  for I := 0 to Self.ControlCount - 1 do
  begin
    if(Self.Controls[I] is TSpeedButton) and
      (TSpeedButton(Self.Controls[I]).GroupIndex = 10) then
    begin
      SpeedButton := TSpeedButton(Self.Controls[I]);
      if SpeedButton.Down then
      begin
        Result := SpeedButton.Tag;
        break;
      end;
    end;
  end;
end;

procedure TLabelPropertyForm.SetLayout(AValue: TTextLayout);
var
  I: Integer;
begin
  I := Get_Tag_of_Button_in_AlignmentLayout mod 3 + (Integer(AValue) * 3);
  Set_Tag_of_Button_in_AlignmentLayout(I);
end;

procedure TLabelPropertyForm.Set_Tag_of_Button_in_AlignmentLayout(AValue: Integer);
var
  I: Integer;
  SpeedButton: TSpeedButton;
  procedure _Proc(S: string);
  begin
    Label_AlignmentLayout.Caption := Label_AlignmentLayout.Hint + ': ' + S;
  end;
begin
  Label_Preview.Alignment := TAlignment(AValue mod 3);
  Label_Preview.Layout := TTextLayout((AValue div 3) mod 3);
  for I := 0 to Self.ControlCount - 1 do
  begin
    if(Self.Controls[I] is TSpeedButton) and
      (TSpeedButton(Self.Controls[I]).GroupIndex = 10) then
    begin
      SpeedButton := TSpeedButton(Self.Controls[I]);
      if SpeedButton.Tag = AValue then
      begin
        SpeedButton.Down := TRUE;
        Label_AlignmentLayout.Caption := Label_AlignmentLayout.Hint + ': ' + SpeedButton.Hint;
        break;
      end;
    end;
  end;
end;

function TLabelPropertyForm.GetAlignment: TAlignment;
begin
  Result := TAlignment(Get_Tag_of_Button_in_AlignmentLayout mod 3);
end;

function TLabelPropertyForm.GetFont: TFont;
begin
  //Result := Label_Preview.Font;
  //Result := FontDialog1.Font;
  Result := FFont;
end;

procedure TLabelPropertyForm.Button_FontClick(Sender: TObject);
begin
  FontDialog1.Font.Assign(FFont);
  if FontDialog1.Execute then
  begin
    SetFont(FontDialog1.Font);
  end;
end;

procedure TLabelPropertyForm.CheckListBox_FontStyleClickCheck(Sender: TObject);
var
  I: Integer;
  FS: TFontStyles;
begin
  FS := [];
  if CheckListBox_FontStyle.Checked[0] then
    FS := FS + [fsBold];
  if CheckListBox_FontStyle.Checked[1] then
    FS := FS + [fsItalic];
  if CheckListBox_FontStyle.Checked[2] then
    FS := FS + [fsStrikeOut];
  if CheckListBox_FontStyle.Checked[3] then
    FS := FS + [fsUnderLine];
  FFont.Style := FS;
  SetFont(FFont);
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TLabelPropertyForm.ColorButton_FontColorColorChanged(Sender: TObject);
begin
  FFont.Color := ColorButton_FontColor.ButtonColor;
  SetFont(FFont);
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TLabelPropertyForm.ComboBox_FontNameChange(Sender: TObject);
var
  I: Integer;
begin
  I := ComboBox_FontName.ItemIndex;
  if I >= 0 then
  begin
    FFont.Name := ComboBox_FontName.Text;
    SetFont(FFont);
    if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
  end;
end;

procedure TLabelPropertyForm.ComboBox_FontSizeChange(Sender: TObject);
var
  I: Integer;
begin
  I := ComboBox_FontSize.ItemIndex;
  if I >= 0 then
  begin
    Font.Size := Integer(ComboBox_FontSize.Items.Objects[I]);
    SetFont(FFont);
    if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
  end;
end;

procedure TLabelPropertyForm.FormCloseQuery(Sender: TObject;
  var CanClose: boolean);
begin
  if(GetText = '')and(ModalResult <> mrCancel)and
    (Application.MessageBox(
      PChar('表示テキストが空のラベル項目は削除されます' + #13#10 + '編集を継続しますか？'),
      '項目は削除されます',
      MB_ICONWARNING or MB_YESNO) = IDYES) then
  begin
    CanClose := FALSE;
  end;
end;

procedure TLabelPropertyForm.FormCreate(Sender: TObject);
var
  I: Integer;
begin
  ComboBox_FontName.Items.Assign(Screen.Fonts);
  FFont := TFont.Create;
  with ComboBox_FontSize do
  begin
    Items.Clear;
    for I := 6 to 96 do
    begin
      if I < 16 then
      begin
        Items.AddObject(IntToStr(I), TObject(I * 2));
        Items.AddObject(IntToStr(I) + '.5', TObject(I * 2 + 1));
      end
      else if (I < 32) or
             ((I < 48)and((I mod 2) = 0)) or
             ((I < 60)and((I mod 3) = 0)) or
             ((I mod 4) = 0) then
        Items.AddObject(IntToStr(I), TObject(I * 2));
    end;
  end;
end;

procedure TLabelPropertyForm.FormDestroy(Sender: TObject);
begin
  FFont.Free;
end;

procedure TLabelPropertyForm.FormHide(Sender: TObject);
begin
  Self.Position := poDesigned;
end;

procedure TLabelPropertyForm.FormShow(Sender: TObject);
begin
  LabeledEdit_Text.SelectAll;
  LabeledEdit_Text.SetFocus;
end;

function TLabelPropertyForm.GetText: string;
begin
  Result := Trim(LabeledEdit_Text.Text);
end;

procedure TLabelPropertyForm.SetAlignment(AValue: TAlignment);
var
  I: Integer;
begin
  I := (Get_Tag_of_Button_in_AlignmentLayout div 3) * 3 + Integer(AValue);
  Set_Tag_of_Button_in_AlignmentLayout(I);
end;

procedure TLabelPropertyForm.SetFont(AValue: TFont);
begin
  FFont.Assign(AValue);

  ComboBox_FontName.ItemIndex := ComboBox_FontName.Items.IndexOf(FFont.Name);
  ComboBox_FontSize.ItemIndex :=
    ComboBox_FontSize.Items.IndexOfObject(TObject(FFont.Size));
  ColorButton_FontColor.ButtonColor:= ColorToRGB(FFont.Color);

  CheckListBox_FontStyle.Checked[0] := fsBold in FFont.Style;
  CheckListBox_FontStyle.Checked[1] := fsItalic in FFont.Style;
  CheckListBox_FontStyle.Checked[2] := fsStrikeOut in FFont.Style;
  CheckListBox_FontStyle.Checked[3] := fsUnderLine in FFont.Style;

  FontDialog1.Font := FFont;
  Label_Preview.Font := FFont;
end;

procedure TLabelPropertyForm.SetText(AValue: string);
begin
  Label_Preview.Caption := AValue;
  LabeledEdit_Text.Text := AValue;
end;

end.

