unit MyDataUtils;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, Windows, Contnrs, LazUTF8Classes;

type

  { TMyDataItemInfo }

  TMyDataItemInfo = packed record
    ID: DWORD;
    HeaderSize: DWORD;
    BodySize: DWORD;
    Check: DWORD;
  end;

  TMyData = class;

  { TMyDataItem }

  TMyDataItem = class
  private
    FData: TMyData;
    FHeader: TStrings;
    FBody: TStream;
  protected
    function GetID: DWORD;
  public
    constructor Create(AData: TMyData);
    destructor Destroy; override;
    procedure Clear;
    function CalcSize: Int64;
    function LoadFromStream(Stream: TStream): Integer;
    function SaveToStream(Stream: TStream): Integer;
    property Header: TStrings read FHeader;
    property Body: TStream read FBody;
  end;

  { TMyDataInfo }

  TMyDataInfo = packed record
    ID: DWORD;
    HeaderSize: DWORD;
    ItemCount: DWORD;
    Check: DWORD;
  end;

  { TMyData }

  TMyData = class
  private
    FID: DWORD;
    FHeader: TStrings;
    FItems: TObjectList;
    function GetCount: Integer;
    function GetItems(Index: Integer): TMyDataItem;
  protected
    function GetID: DWORD; virtual;
  public
    constructor Create(AID: DWORD = 0);
    destructor Destroy; override;
    procedure Clear;
    function LoadFromStream(Stream: TStream): Int64;
    function SaveToStream(Stream: TStream): Int64;
    function LoadFromFile(FileName: string): Boolean;
    function SaveToFile(FileName: string): Boolean;
    property Header: TStrings read FHeader;
    property Items[Index: Integer]: TMyDataItem read GetItems;
    property Count: Integer read GetCount;
  end;

implementation

{ TMyDataItem }

function TMyDataItem.GetID: DWORD;
begin
  if FData = nil then
    Result := 0
  else
    Result := FData.FID;
end;

constructor TMyDataItem.Create(AData: TMyData);
begin
  FHeader := TStringList.Create;
  FBody := TMemoryStream.Create;
  FData := AData;
  AData.FItems.Add(Self);
end;

destructor TMyDataItem.Destroy;
begin
  FBody.Free;
  FHeader.Free;
  inherited Destroy;
end;

procedure TMyDataItem.Clear;
begin
  FHeader.Clear;
  (FBody as TMemoryStream).Clear;
end;

function TMyDataItem.CalcSize: Int64;
var
  HeaderStream: TMemoryStream;
begin
  Result := sizeof(TMyDataItemInfo);

  HeaderStream := TMemoryStream.Create;
  try
    FHeader.SaveToStream(HeaderStream);
    Inc(Result, HeaderStream.Size);
  finally
    HeaderStream.Free;
  end;

  Inc(Result, FBody.Size);
end;

function TMyDataItem.LoadFromStream(Stream: TStream): Integer;
var
  HeaderStream: TMemoryStream;
  Info: TMyDataItemInfo;
  IDone: Int64;
  bCompleted: Boolean;
begin
  Result := 0;
  Clear;
  bCompleted := FALSE;
  IDone := Stream.Read(Info, sizeof(Info));
  Inc(Result, IDone);
  if IDone <> sizeof(Info) then
  else if Info.ID <> Self.GetID then
  else if (Info.ID xor Info.HeaderSize xor Info.BodySize) <> Info.Check then
  else if(Stream.Size - Stream.Position) >= (Info.HeaderSize + Info.BodySize) then
  begin
    HeaderStream := TMemoryStream.Create;
    try
      if Info.HeaderSize = 0 then
        IDone := 0
      else
      begin
        IDone := HeaderStream.CopyFrom(Stream, Info.HeaderSize);
        Inc(Result, IDone);
      end;
      if IDone <> Info.HeaderSize then
      else
      begin
        HeaderStream.Position := 0;
        FHeader.LoadFromStream(HeaderStream);

        if Info.BodySize = 0 then
          IDone := 0
        else
          IDone := FBody.CopyFrom(Stream, Info.BodySize);

        Inc(Result, IDone);
        if IDone = Info.BodySize then
        begin
          Inc(Result, IDone);
          FBody.Position := 0;
          bCompleted := TRUE;
        end;
      end;
    finally
      HeaderStream.Free;
    end;
  end;
  if bCompleted = FALSE then
    Result := -Result;
end;

function TMyDataItem.SaveToStream(Stream: TStream): Integer;
var
  HeaderStream: TMemoryStream;
  Info: TMyDataItemInfo;
  IDone: Int64;
  bCompleted: Boolean;
begin
  Result := 0;
  bCompleted := FALSE;
  HeaderStream := TMemoryStream.Create;
  try
    FHeader.SaveToStream(HeaderStream);
    if(sizeof(Info) + HeaderStream.Size + FBody.Size) > MaxInt then
    else
    begin
      Info.ID := Self.GetID;
      Info.HeaderSize := HeaderStream.Size;
      Info.BodySize := FBody.Size;
      Info.Check := Info.ID xor Info.HeaderSize xor Info.BodySize;
      IDone := Stream.Write(Info, sizeof(Info));
      Inc(Result, IDone);
      if IDone <> sizeof(Info) then
      else
      begin
        if HeaderStream.Size = 0 then
          IDone := 0
        else
        begin
          HeaderStream.Position := 0;
          IDone := Stream.CopyFrom(HeaderStream, HeaderStream.Size);
          Inc(Result, IDone);
        end;
        if IDone <> HeaderStream.Size then
        else
        begin
          if FBody.Size = 0 then
            IDone := 0
          else
          begin
            FBody.Position := 0;
            IDone := Stream.CopyFrom(FBody, FBody.Size);
            Inc(Result, IDone);
          end;
          bCompleted := IDone = FBody.Size;
        end;
      end;
    end;
  finally
    HeaderStream.Free;
  end;
  if bCompleted = FALSE then
    Result := -Result;
end;

{ TMyData }

function TMyData.GetItems(Index: Integer): TMyDataItem;
begin
  Result := FItems[Index] as TMyDataItem;
end;

function TMyData.GetCount: Integer;
begin
  Result := FItems.Count;
end;

function TMyData.GetID: DWORD;
begin
  Result := FID;
end;

constructor TMyData.Create(AID: DWORD);
begin
  FHeader := TStringList.Create;
  FItems := TObjectList.Create(TRUE);
  if AID = 0 then
    FID := PDWORD(PChar('KWBD'))^
  else
    FID := AID;
end;

destructor TMyData.Destroy;
begin
  FItems.Free;
  FHeader.Free;
  inherited Destroy;
end;

procedure TMyData.Clear;
begin
  FItems.Clear;
end;

function TMyData.LoadFromStream(Stream: TStream): Int64;
var
  HeaderStream: TMemoryStream;
  Info: TMyDataInfo;
  IDone: Int64;
  I: Integer;
  Item: TMyDataItem;
  bCompleted: Boolean;
begin
  Result := 0;
  Clear;
  bCompleted := FALSE;
  IDone := Stream.Read(Info, sizeof(Info));
  if IDone <> sizeof(Info) then
  else if Info.ID <> Self.GetID then
  else if(Info.ID xor Info.HeaderSize xor Info.ItemCount) <> Info.Check then
  else
  begin
    HeaderStream := TMemoryStream.Create;
    try
      if Info.HeaderSize = 0 then
        IDone := 0
      else
      begin
        IDone := HeaderStream.CopyFrom(Stream, Info.HeaderSize);
        Inc(Result, IDone);
      end;
      if IDone <> Info.HeaderSize then
      else
      begin
        HeaderStream.Position := 0;
        FHeader.LoadFromStream(HeaderStream);

        for I := 0 to Info.ItemCount - 1 do
        begin
          Item := TMyDataItem.Create(Self);
          IDone := Item.LoadFromStream(Stream);
          if IDone > 0 then
            Inc(Result, IDone)
          else
          begin
            Inc(Result, Abs(IDone));
            break;
          end;
          if I = (Info.ItemCount - 1) then
            bCompleted := TRUE;
        end;
      end;
    finally
      HeaderStream.Free;
    end;
  end;
  if bCompleted = FALSE then
    Result := -Result;
end;

function TMyData.SaveToStream(Stream: TStream): Int64;
var
  HeaderStream: TMemoryStream;
  Info: TMyDataInfo;
  IDone: Int64;
  I: Integer;
  Item: TMyDataItem;
  bCompleted: Boolean;
begin
  Result := 0;
  bCompleted := FALSE;
  HeaderStream := TMemoryStream.Create;
  try
    FHeader.SaveToStream(HeaderStream);
    if(sizeof(Info) + HeaderStream.Size) > MaxInt then
    else
    begin
      Info.ID := Self.GetID;
      Info.HeaderSize := HeaderStream.Size;
      Info.ItemCount := FItems.Count;
      Info.Check := Info.ID xor Info.HeaderSize xor Info.ItemCount;
      IDone := Stream.Write(Info, sizeof(Info));
      Inc(Result, IDone);
      if IDone <> sizeof(Info) then
      else
      begin
        HeaderStream.Position := 0;
        IDone := Stream.CopyFrom(HeaderStream, HeaderStream.Size);
        Inc(Result, IDone);
        if IDone <> HeaderStream.Size then
        else
        begin
          for I := 0 to FItems.Count - 1 do
          begin
            Item := TMyDataItem(FItems[I]);
            IDone := Item.SaveToStream(Stream);
            if IDone > 0 then
              Inc(Result, IDone)
            else
            begin
              Inc(Result, Abs(IDone));
              break;
            end;
            if I = (FItems.Count - 1) then
              bCompleted := TRUE;
          end;
        end;
      end;
    end;
  finally
    HeaderStream.Free;
  end;
  if bCompleted = FALSE then
    Result := -Result;
end;

function TMyData.LoadFromFile(FileName: string): Boolean;
var
  FS: LazUTF8Classes.TFileStreamUTF8;
  IDone: Int64;
begin
  FS := LazUTF8Classes.TFileStreamUTF8.Create(FileName, fmOpenRead or fmShareDenyNone);
  try
    IDone := Self.LoadFromStream(FS);
    Result := IDone > 0;
  finally
    FS.Free;
  end;
end;

function TMyData.SaveToFile(FileName: string): Boolean;
var
  FS: LazUTF8Classes.TFileStreamUTF8;
  IDone: Int64;
begin
  FS := LazUTF8Classes.TFileStreamUTF8.Create(FileName, fmCreate);
  try
    IDone := Self.SaveToStream(FS);
    Result := IDone > 0;
  finally
    FS.Free;
  end;
end;

end.

