unit ShapePropertyUnit;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  StdCtrls, Types, FPCanvas, LCLType;

type

  { TShapePropertyForm }

  TShapePropertyForm = class(TForm)
    Bevel1: TBevel;
    Bevel2: TBevel;
    Button1: TButton;
    Button2: TButton;
    ColorButton_BrushColor: TColorButton;
    ColorButton_PenColor: TColorButton;
    ComboBox_PenStyle: TComboBox;
    ComboBox_PenWidth: TComboBox;
    ComboBox_Shape: TComboBox;
    ComboBox_BrushStyle: TComboBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label_PenColor: TLabel;
    Label_PenColor1: TLabel;
    Label_PenStyle: TLabel;
    Label_PenStyle1: TLabel;
    Label_PenWidth: TLabel;
    procedure Button1Click(Sender: TObject);
    procedure ColorButton_BrushColorColorChanged(Sender: TObject);
    procedure ColorButton_PenColorColorChanged(Sender: TObject);
    procedure ComboBox_BrushStyleDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure ComboBox_BrushStyleSelect(Sender: TObject);
    procedure ComboBox_PenStyleDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure ComboBox_PenStyleSelect(Sender: TObject);
    procedure ComboBox_PenWidthDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure ComboBox_PenWidthSelect(Sender: TObject);
    procedure ComboBox_ShapeDrawItem(Control: TWinControl; Index: Integer;
      ARect: TRect; State: TOwnerDrawState);
    procedure ComboBox_ShapeSelect(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    FShape: Integer;
    FPen: TPen;
    FBrush: TBrush;
    FOnPropertyChange: TNotifyEvent;
    function GetBrush: TBrush;
    function GetPen: TPen;
    function GetShape: Integer;
    procedure SetBrush(AValue: TBrush);
    procedure SetPen(AValue: TPen);
    procedure SetShape(AValue: Integer);
    { private declarations }
  public
    { public declarations }
    property Pen: TPen read GetPen write SetPen;
    property Brush: TBrush read GetBrush write SetBrush;
    property Shape: Integer read GetShape write SetShape;
    property OnPropertyChange: TNotifyEvent read FOnPropertyChange write FOnPropertyChange;
  end;

var
  ShapePropertyForm: TShapePropertyForm;

implementation

{$R *.lfm}

{ TShapePropertyForm }

procedure TShapePropertyForm.Button1Click(Sender: TObject);
begin
  //Shape1.Pen.Style:=;
end;

procedure TShapePropertyForm.ColorButton_BrushColorColorChanged(Sender: TObject);
var
  BrushStyle_Saved: TBrushStyle;
begin
  ComboBox_BrushStyle.Invalidate;
  BrushStyle_Saved := FBrush.Style;
  FBrush.Color := ColorButton_BrushColor.ButtonColor;
  if BrushStyle_Saved = bsClear then
    FBrush.Style := bsSolid
  else
    FBrush.Style := BrushStyle_Saved;
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.ColorButton_PenColorColorChanged(Sender: TObject);
begin
  ComboBox_PenStyle.Invalidate;
  ComboBox_PenWidth.Invalidate;
  FPen.Color := ColorButton_PenColor.ButtonColor;
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.ComboBox_ShapeDrawItem(Control: TWinControl; Index: Integer;
  ARect: TRect; State: TOwnerDrawState);
var
  S: string;
  I: Integer;
  TS: TTextStyle;
begin
  S := ComboBox_Shape.Items[Index];
  I := Pos(' ', S);
  //Shape1.Shape := TShapeType(StrToIntDef(Copy(S, 1, I - 1), 4));
  with ComboBox_Shape do
  begin
    Canvas.FillRect(ARect);
    FillChar(TS, sizeof(TS), 0);
    TS.Layout := tlCenter;
    Canvas.TextRect(ARect, ARect.Left + 2, ARect.Top, Copy(S, I + 1, MaxInt), TS);
  end;
end;

procedure TShapePropertyForm.ComboBox_BrushStyleDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
var
  I, R, G, B: Integer;
  TS: TTextStyle;
begin
  with ComboBox_BrushStyle do
  begin
    if odSElected in State then
    begin
      Canvas.Brush.Color := clHighlight;
      Canvas.Font.Color := clHighlightText;
    end
    else
    begin
      I := ColorToRGB(ColorButton_BrushColor.ButtonColor);
      R := I and $ff;
      G := (I shr 8) and $ff;
      B := (I shr 16) and $ff;
      if (R + G + B) > $240 then
      begin
        Canvas.Brush.Color := clBlack;
        Canvas.Font.Color := clWhite;
      end
      else
      begin
        Canvas.Brush.Color := clWindow;
        Canvas.Font.Color := clWindowText;
      end;
    end;
    Canvas.FillRect(ARect);

    Canvas.Brush.Color := ColorButton_BrushColor.ButtonColor;
    Canvas.Brush.Style := TFPBrushStyle(StrToIntDef(ComboBox_BrushStyle.Items[Index][1], Integer(psSolid)));

    if Canvas.Brush.Style = bsClear then
    begin
      FillChar(TS, sizeof(TS), 0);
      TS.Layout := tlCenter;
      Canvas.TextRect(ARect, ARect.Left + 2, ARect.Top, '(none)', TS);
    end
    else
    begin
      Canvas.FillRect(ARect.Left + 4, ARect.Top + 4, ARect.Right - 4, ARect.Bottom - 4);
    end;
  end;
end;

procedure TShapePropertyForm.ComboBox_BrushStyleSelect(Sender: TObject);
begin
  ComboBox_BrushStyle.Invalidate;
  FBrush.Style := TFPBrushStyle(StrToIntDef(Copy(ComboBox_BrushStyle.Text, 1, 1), Integer(bsSolid)));
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.ComboBox_PenStyleDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
var
  I, R, G, B: Integer;
  TS: TTextStyle;
begin
  with ComboBox_PenStyle do
  begin
    Canvas.Pen.Color := ColorButton_PenColor.ButtonColor;
    Canvas.Pen.Style := TFPPenStyle(StrToIntDef(ComboBox_PenStyle.Items[Index][1], Integer(psSolid)));
    Canvas.Pen.Width := StrToIntDef(ComboBox_PenWidth.Text, 1);
    if odSelected in State then
    begin
      Canvas.Pen.Color := clHighlightText;
      Canvas.Brush.Color := clHighlight;
    end
    else
    begin
      I := ColorToRGB(Canvas.Pen.Color);
      R := I and $ff;
      G := (I shr 8) and $ff;
      B := (I shr 16) and $ff;
      if (R + G + B) > $240 then
        Canvas.Brush.Color := clBlack
      else
        Canvas.Brush.Color := clWhite;
    end;
    Canvas.FillRect(ARect);
    if Canvas.Pen.Style = psClear then
    begin
      FillChar(TS, sizeof(TS), 0);
      TS.Layout := tlCenter;
      Canvas.Font.Color := Canvas.Pen.Color;
      Canvas.TextRect(ARect, ARect.Left + 2, ARect.Top, '(none)', TS);
    end
    else
    begin
      Canvas.MoveTo(ARect.Left, (ARect.Top + ARect.Bottom) div 2);
      Canvas.LineTo(ARect.Right, (ARect.Top + ARect.Bottom) div 2);
    end;
  end;
end;

procedure TShapePropertyForm.ComboBox_PenStyleSelect(Sender: TObject);
begin
  ComboBox_PenWidth.Invalidate;
  ComboBox_PenStyle.Invalidate;
  FPen.Style := TFPPenStyle(StrToIntDef(Copy(ComboBox_PenStyle.Text, 1, 1), Integer(psSolid)));
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.ComboBox_PenWidthDrawItem(Control: TWinControl;
  Index: Integer; ARect: TRect; State: TOwnerDrawState);
var
  I, R, G, B: Integer;
  TS: TTextStyle;
begin
  with ComboBox_PenWidth do
  begin
    Canvas.Pen.Color := ColorButton_PenColor.ButtonColor;
    Canvas.Pen.Style := TFPPenStyle(StrToIntDef(Copy(ComboBox_PenStyle.Text, 1, 1), Integer(psSolid)));
    //Canvas.Pen.Width := StrToIntDef(ComboBox_PenWidth.Text, 1);
    Canvas.Pen.Width := StrToIntDef(Copy(ComboBox_PenWidth.Items[Index], 1, 1), 1);
    if odSelected in State then
    begin
      Canvas.Pen.Color := clHighlightText;
      Canvas.Brush.Color := clHighlight;
    end
    else
    begin
      I := ColorToRGB(Canvas.Pen.Color);
      R := I and $ff;
      G := (I shr 8) and $ff;
      B := (I shr 16) and $ff;
      if (R + G + B) > $240 then
        Canvas.Brush.Color := clBlack
      else
        Canvas.Brush.Color := clWhite;
    end;
    Canvas.FillRect(ARect);
    Canvas.MoveTo(ARect.Left, (ARect.Top + ARect.Bottom) div 2);
    Canvas.LineTo(ARect.Right, (ARect.Top + ARect.Bottom) div 2);
  end;
end;

procedure TShapePropertyForm.ComboBox_PenWidthSelect(Sender: TObject);
begin
  ComboBox_PenWidth.Invalidate;
  ComboBox_PenStyle.Invalidate;
  FPen.Width := StrToIntDef(ComboBox_PenWidth.Text, 1);
  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.ComboBox_ShapeSelect(Sender: TObject);
var
  S: string;
  I: Integer;
begin
  S := ComboBox_Shape.Text;
  I := Pos(' ', S);
  FShape := StrToIntDef(Copy(S, 1, I - 1), 4);

  if Assigned(FOnPropertyChange) then FOnPropertyChange(Self);
end;

procedure TShapePropertyForm.FormCreate(Sender: TObject);
begin
  //FShape: Integer;
  FPen := TPen.Create;
  FBrush := TBrush.Create;
end;

procedure TShapePropertyForm.FormDestroy(Sender: TObject);
begin
  FBrush.Free;
  FPen.Free;
end;

procedure TShapePropertyForm.FormHide(Sender: TObject);
begin
  Self.Position := poDesigned;
end;

function TShapePropertyForm.GetBrush: TBrush;
begin
  //Result := Shape1.Brush;
  Result := FBrush;
end;

function TShapePropertyForm.GetPen: TPen;
begin
  //Result := Shape1.Pen;
  Result := FPen;
end;

function TShapePropertyForm.GetShape: Integer;
begin
  //Result := Shape1.Shape;
  Result := FShape;
end;

procedure TShapePropertyForm.SetBrush(AValue: TBrush);
var
  I, K: Integer;
  S: string;
begin
  FBrush.Assign(AValue);
  //FBrush.Style := AValue.Style; // こっちではダメな場合がある？
  ColorButton_BrushColor.ButtonColor := FBrush.Color;
  FBrush.Style := AValue.Style; // LCLの低頻度のバグ対策？
  // TBrush.Color プロパティへのアクセスが Style に影響する場合があるのか？

  ComboBox_BrushStyle.ItemIndex := -1;
  for I := 0 to ComboBox_BrushStyle.Items.Count - 1 do
  begin
    S := Copy(ComboBox_BrushStyle.Items[I], 1, 1);
    K := StrToIntDef(S, -1);
    if K = Integer(FBrush.Style) then
    begin
      ComboBox_BrushStyle.ItemIndex := I;
      break;
    end;
  end;
end;

procedure TShapePropertyForm.SetPen(AValue: TPen);
var
  I: Integer;
  S: string;
begin
  FPen.Assign(AValue);
  ColorButton_PenColor.ButtonColor := FPen.Color;

  //ComboBox_PenStyle := Shape1.Pen.Style;
  for I := 0 to ComboBox_PenStyle.Items.Count - 1 do
  begin
    S := Copy(ComboBox_PenStyle.Items[I], 1, 1);
    if S = IntToStr(Integer(FPen.Style)) then
    begin
      ComboBox_PenStyle.ItemIndex := I;
      break;
    end;
    if I = (ComboBox_PenStyle.Items.Count - 1) then
      ComboBox_PenStyle.ItemIndex := -1;
  end;

  if ComboBox_PenStyle.ItemIndex = -1 then
  for I := 0 to ComboBox_PenStyle.Items.Count - 1 do
  begin
    S := Copy(ComboBox_PenStyle.Items[I], 1, 1);
    if S = IntToStr(Integer(psSolid)) then
    begin
      ComboBox_PenStyle.ItemIndex := I;
      break;
    end;
  end;

  //ComboBox_PenWidth := Shape1.Pen.Width;
  for I := 0 to ComboBox_PenWidth.Items.Count - 1 do
  begin
    S := Copy(ComboBox_PenWidth.Items[I], 1, 1);
    if S = IntToStr(FPen.Width) then
    begin
      ComboBox_PenWidth.ItemIndex := I;
      break;
    end;
    if I = (ComboBox_PenWidth.Items.Count - 1) then
      ComboBox_PenWidth.ItemIndex := -1;
  end;
  if ComboBox_PenWidth.ItemIndex = -1 then
  for I := 0 to ComboBox_PenWidth.Items.Count - 1 do
  begin
    S := Copy(ComboBox_PenWidth.Items[I], 1, 1);
    if S = IntToStr(1) then
    begin
      ComboBox_PenWidth.ItemIndex := I;
      break;
    end;
  end;
end;

procedure TShapePropertyForm.SetShape(AValue: Integer);
var
  I: Integer;
  S: string;
begin
  FShape := Integer(AValue);
  for I := 0 to ComboBox_Shape.Items.Count  - 1 do
  begin
    S := ComboBox_Shape.Items[I];
    if AValue = StrToIntDef(Copy(S, 1, Pos(' ', S) - 1), 4) then
    begin
      ComboBox_Shape.ItemIndex := I;
      break;
    end;
  end;
end;

end.

